'use strict';


angular.module('fadel',[])
    .config(['$locationProvider', function($locationProvider) {
        $locationProvider.html5Mode(true);
    }])
    .run(['$document', function($document) {
        $document[0].body.style.visibility = 'visible';
    }])
    .constant('formatTypeCode', 20)
    .constant('territoryTypeCode', 30)
    .controller('AppController', ['$timeout', '$q', '$scope', '$location', '$window', 'getAssetClearanceSearchCriteria', 'requestRightAttributes', 'requestAssetClearance', 'requestAssetThumbnail', 'requestAssetRightsAvailabilityAnswers', 'formatTypeCode', 'territoryTypeCode', 'global', function($timeout, $q, $scope, $location, $window, getAssetClearanceSearchCriteria, requestRightAttributes, requestAssetClearance, requestAssetThumbnail, requestAssetRightsAvailabilityAnswers, formatTypeCode, territoryTypeCode, global) {
        var locationSearch = $location.search();
        var redirectBoxUrl = locationSearch['redirect_to_box_url'];
        $scope.fileId = locationSearch['file_id'];
        $scope.fileName = locationSearch['file_name'];
        $scope.currentClearance = {};
        $scope.clearances = [];

        function getRightAttributes(callback){
            $q.all([requestRightAttributes(formatTypeCode), requestRightAttributes(territoryTypeCode)]).then(function(attributes){
                $scope.formatRightAttributes = attributes[0];
                $scope.territoryRightAttributes = attributes[1];
                $scope.attributes = {};
                $scope.attributes.format = $scope.formatRightAttributes.length == 0 ? null : $scope.formatRightAttributes;
                $scope.attributes.territory = $scope.territoryRightAttributes.length == 0 ? null : $scope.territoryRightAttributes;
                callback(true);
            }).catch(function(response){
                if(response.status == 403){
                    callback(false);
                    $scope.appLoaded = false;
                    $scope.authError = true;
                    $scope.authErrorMsg = response.status + ': ' + response.statusText;
                } else if(response.status == 500)  {
                    callback(false);
                    $scope.appLoaded = false;
                    $scope.authError = true;
                    $scope.authErrorMsg = response.status + ': ' + response.statusText;
                } else {
                    showError(response);
                }
            });
        }


        function showError(response){
            console.log('ERROR Server request failed with response: ' + response + '  status code: ' + response.status + ', message: ' + (response.data || ''));
        }

        $scope.setError = function(message){
            $scope.error = message;
        }

        getRightAttributes(function(){
            $scope.viewType = "clearance";
        });

        $scope.close = function(){
            $window.location.href = redirectBoxUrl;
        }

        $scope.closeModal = function(){
            $scope.clearances = [];
            $(".btn-default").closest("coral-dialog").find(".coral-Dialog-closeButton").trigger("click");
        }

        $scope.getDates = {};
        $scope.getClearance = function(){
            $scope.fadelSelectionAllNamesArr = window.fadelSelectionAllNamesArr;
            $scope.fadelSelectionAllUuid = window.fadelSelectionAllUuid;
            $scope.closeExceptionsDetails();
            $scope.clearance = null;
            var fromDate = $scope.getDates.from();
            var toDate = $scope.getDates.to();
            $scope.formFillError = false;
            $scope.datesError = false;
            if(!$scope.attributes.format || !$scope.attributes.territory || !fromDate || !toDate){
                $scope.formFillError = true;
            }
            else if (fromDate > toDate) {
                $scope.datesError = true;
            }
            else {
                var territory = $scope.attributes.territory;
                territory = Array.isArray(territory) ? territory : [territory];
                var format = $scope.attributes.format;
                format = Array.isArray(format) ? format : [format];
                var searchCriteria = getAssetClearanceSearchCriteria($scope.fileId, $scope.fileName, format, territory, fromDate.toDate(), toDate.toDate());
                requestAssetClearance(searchCriteria).then(function(clearances){
                    $scope.searchCriteria = searchCriteria;
                    $scope.clearances = [];
                    var i = 0;
                    angular.forEach(clearances, function (clearance) {
                        if (clearance.thumbNailUri) {
                            requestAssetThumbnail(i, clearance.thumbNailUri).then(function(ret){
                                var idx = ret[0];
                                var thumbnail = ret[1];
                                var clearance = $scope.clearances[idx];
                                clearance['thumbnail'] = thumbnail;
                            })
                        }
                        $scope.clearances.push ({
                            idx: i,
                            uuid: clearance.asset.assetExtId,
                            typeName : clearance.typeName,
                            fileName : clearance.asset.name,
                            availableAnswerListUri : clearance.availableAnswerListUri,
                            notAvailableAnswerListUri : clearance.notAvailableAnswerListUri,
                        });
                        i += 1;
                    });
                    window.fadelClearances = $scope.clearances;


                }).catch(function(response){
                    $scope.clearance = {typeName : 'COULD NOT GET DATA'};
                });
            }
        }

        $scope.closeExceptionsDetails = function(clearance) {
            if (clearance) {
                clearance.exceptionsDetails = null;
            }
        }

        $scope.getExceptionsDetails = function(clearance){
            $q.all([requestAssetRightsAvailabilityAnswers(clearance.availableAnswerListUri, $scope.searchCriteria, clearance, true),
                requestAssetRightsAvailabilityAnswers(clearance.notAvailableAnswerListUri, $scope.searchCriteria, clearance, false)]).then(function(results){
                var availableResults = results[0][0];
                var notAvailableResults = results[1][0];
                var clearance = results[0][1];
                var answers = availableResults.concat(notAvailableResults);
                var formats = [];
                var territories = [];
                var availability = [];
                for(var i = 0; i < answers.length; i++){
                    var answer = answers[i];
                    var territoryIndex = territories.indexOf(answer.territory);
                    if(territoryIndex == -1){
                        territoryIndex = territories.length;
                        territories.push(answer.territory);
                        availability.push([]);
                    }
                    var formatIndex = formats.indexOf(answer.format);
                    if(formatIndex == -1){
                        formatIndex = formats.length;
                        formats.push(answer.format);
                    }
                    availability[territoryIndex][formatIndex] = answer.available;
                }
                $scope.currentClearance = $scope.clearances[clearance.idx];
                $scope.clearances[clearance.idx]["exceptionsDetails"] = {
                    formats : formats,
                    territories : territories,
                    availability : availability
                }
            }).catch(function(response){
                showError(response);
            });
        }

        //------------------init app-----------------------

        getRightAttributes(function(){
            $scope.viewType =  "clearance";
            $scope.appLoaded = true;
        });

    }])
    .directive('hover', ["$window", function($window) {
        return {
            restrict: 'E',
            transclude : true,
            link: function(scope, element, attrs) {
                if(!scope.enabled){
                    return;
                }
                var item = element.find('div');
                window.hoverClick = function(idx){
                    scope.$apply(function(){
                        var clearance = window.fadelClearances [idx];
                        scope.click(clearance, idx);
                        item.popover('hide');
                    })
                }
                item.popover({content : '<div>To view details, <a href onclick="hoverClick(' + scope.clearance.idx + ')">click here</a></div>', container: item, trigger: "hover", html: true, placement : "top"});
            },
            scope: {click : '=', enabled : '=', clearance: '='},
            template: '<ng-transclude></ng-transclude>'
        };
    }])
    .directive('dateInput', function() {
        return {
            restrict: 'E',
            link: function(scope, element, attrs) {
                var dateTimePicker = element;
                dateTimePicker.datetimepicker({format : 'MM/DD/YYYY'});
                scope.getDate = function (){
                    return dateTimePicker.data("DateTimePicker").date();
                };
            },
            replace : true,
            scope: {getDate : '=', placeholder : '@'},
            template: '<div class="input-group date"><input type="text" class="form-control" placeholder="{{placeholder}}"/><span class="input-group-addon"><span class="glyphicon glyphicon-calendar"></span></span></div>'
        };
    })
    .factory('global', ['$window', function() {
        return {};
    }])
    .factory('getAssetClearanceSearchCriteria', ['formatTypeCode', 'territoryTypeCode', function(formatTypeCode, territoryTypeCode) {
        return function(assetId, assetName, formatAttributes, territoryAttributes, dateFrom, dateTo){
            var selected = {};
            selected[formatTypeCode] = {};
            for (var i=0; i < formatAttributes.length; i++) {
                selected[formatTypeCode][formatAttributes[i].rightRelationshipId] = {};
            }
            selected[territoryTypeCode] = {};
            for (var i=0; i < territoryAttributes.length; i++) {
                selected[territoryTypeCode][territoryAttributes[i].rightRelationshipId] = {};
            }
            var partial = {};
            partial[formatTypeCode] = {};
            partial[territoryTypeCode] = {};
            var all = {};
            all[formatTypeCode] = "false";
            all[territoryTypeCode] = "false";

            var selectedAssets = {};

            for(var i = 0; i < window.fadelSelectionAllNamesArr.length; i++) {
                var selUuid = window.fadelSelectionAllUuid[i];
                var selName = window.fadelSelectionAllNamesArr[i];
                selectedAssets[selUuid] = {
                    "externalIdentifier": selUuid,
                    "name": selName
                };
            };

            var data = {
                "assets": {},
                "rights": {
                    "selected": selected,
                    "partial": partial,
                    "all": all
                },
                "inDateFrom": dateFrom.getTime(),
                "outDateFrom": dateTo.getTime(),
                "externalAssets": {
                    "selected": selectedAssets,
                    "all": "false"
                },
                "rightTypeLst": [
                    {
                        "lookupType": "FPI_RIGHT_DEFINITION_TYPES",
                        "fieldOrder": 10,
                        "lookupCode": '' + formatTypeCode,
                        "displayName": "Format"
                    },
                    {
                        "lookupType": "FPI_RIGHT_DEFINITION_TYPES",
                        "fieldOrder": 20,
                        "lookupCode": '' + territoryTypeCode,
                        "displayName": "Territory"
                    }
                ]
            };
            return data;
        }
    }])
    .factory('requestAssetRightsAvailabilityAnswers', ['api', '$q', 'formatTypeCode', 'territoryTypeCode', function(api, $q, formatTypeCode, territoryTypeCode) {
        return function(uri, searchCriteria, clearance, available){
            if(!uri){
                return $q.when([[], clearance]);
            }
            var uuid = clearance.uuid;
            searchCriteria["externalAssets"]["selected"] = {};
            searchCriteria["externalAssets"]["selected"][uuid] = {
                "externalIdentifier": clearance.uuid
            };
            return api.do(uri, searchCriteria, 'POST', 'application/json').then(function(response){
                var results = [];
                var answers = response.data.assetAnswer;
                for(var i = 0; i < answers.length; i++){
                    var answer = answers[i].answer;
                    results.push({
                        format : answer[formatTypeCode].right.description,
                        territory : answer[territoryTypeCode].right.description,
                        available : available
                    });
                }
                return [results, clearance];
            });
        }
    }])
    .factory('requestAssetClearance', ['api', function(api) {
        return function(searchCriteria){
            return api.do(api.baseUri + 'clearanceWS/assetClearance', searchCriteria, 'POST', 'application/json').then(function(response){
                var results = "";
                results = response.data.clearance;
                if (results) {
                    var modResults = JSON.stringify(results);
                    modResults = modResults.replace(/(https?:\/\/)([^\/])*(\/ipmSuite5\/ws\/V1\/)/g,"");
                    modResults = JSON.parse(modResults);
                    return modResults;
                }
            });
        }
    }])
    .factory('requestRightAttributes', ['api', function(api) {
        return function(typeCode){

            var results = [];
            function addAttributes(parentNames, attributes){
                for(var i = 0; i < attributes.length; i++){
                    var attribute = attributes[i];
                    var names = parentNames.slice();
                    names.push(attribute.right.description);
                    results.push({
                        rightRelationshipId : attribute.rightRelationshipId,
                        name : names.join(' > ')
                    });
                    addAttributes(names, attribute.childrenLst);
                }
            }
            return api.do(api.baseUri + 'rightWS/rightAttributes/' + typeCode, {}, 'POST', 'application/x-www-form-urlencoded').then(function(response){
                addAttributes([], response.data.attribute);
                return results;
            })
        }
    }])
    .factory('requestAssetThumbnail', ['api', function(api) {
        return function(idx, uri){
            return api.do(uri, null, 'GET', 'application/x-www-form-urlencoded').then(function(response){
                return [idx, response.data.thumbnNail];
            })
        }
    }])
    .factory('api', ['$http', 'global', function($http, global) {
        return {
            baseUri : '',
            do : function(uri, data, method, contentType){
                return $http({
                    method : 'POST',
                    url: '/bin/assetClearance',
                    data : {
                        uri : uri,
                        data : data,
                        method : method,
                        contentType : contentType
                    }
                });
            }
        }
    }]);

angular.element(document).ready(function() {
    angular.bootstrap(document, ["fadel"]);
});
